//
//  AppDelegate.m
//  CHNYC PlugIn Demo
//
//  Created by Jonathan Nathan on 10/11/11.
//  Copyright 2011 JNSoftware LLC. All rights reserved.
//

#import "AppDelegate.h"
#import <CHNYCPlugInDemo/CHNYCPlugInDemo.h>

static NSUserDefaults *standardUserDefaults = nil;

@implementation AppDelegate

@synthesize window, arrayController;

+ (void)initialize {
	static BOOL isInitialzed = NO;
	if (!isInitialzed) {
		standardUserDefaults = [NSUserDefaults standardUserDefaults];
		NSDictionary *appDefaults = [NSDictionary dictionaryWithObjectsAndKeys:
									 [NSNumber numberWithInteger:0], @"lastSelectionIndex", 
									 nil];
		[standardUserDefaults registerDefaults:appDefaults];
		[standardUserDefaults synchronize];
		isInitialzed = YES;
	}
}

- (void)applicationDidFinishLaunching:(NSNotification *)notification {
	[CHNYCPlugIn loadPlugIns];
#if DEBUG
	NSLog(@"loaded plugins: %@", [[[CHNYCPlugIn loadedPlugIns] valueForKey:@"identifier"] componentsJoinedByString:@", "]);
#endif
	[[NSAppleEventManager sharedAppleEventManager] setEventHandler:self andSelector:@selector(handleGetSdefEvent:withReplyEvent:) forEventClass:'ascr' andEventID:'gsdf'];
	[arrayController setContent:[CHNYCPlugIn loadedPlugIns]];
	[arrayController setSortDescriptors:[NSArray arrayWithObjects:[[[NSSortDescriptor alloc] initWithKey:@"displayName" ascending:YES] autorelease], nil]];
	[arrayController rearrangeObjects];
	[arrayController setSelectionIndex:[standardUserDefaults integerForKey:@"lastSelectionIndex"]];
	if (![standardUserDefaults objectForKey:[NSString stringWithFormat:@"NSWindow Frame %@", [window frameAutosaveName]]]) [window center];
	[window makeKeyAndOrderFront:nil];
}

- (void)applicationWillTerminate:(NSNotification *)notification {
	[standardUserDefaults setInteger:[arrayController selectionIndex] forKey:@"lastSelectionIndex"];
	[standardUserDefaults synchronize];
}

- (BOOL)applicationShouldTerminateAfterLastWindowClosed:(NSApplication *)application {
	return YES;
}

- (IBAction)sayHello:(id)sender {
	CHNYCPlugIn *plugIn = (CHNYCPlugIn *)[[arrayController selectedObjects] lastObject];
	if (plugIn) [plugIn sayHello];
}

- (void)handleGetSdefEvent:(NSAppleEventDescriptor *)event withReplyEvent:(NSAppleEventDescriptor *)replyEvent {
	NSAppleEventDescriptor *combinedSdefDescriptor = nil;
	NSError *error = nil;
	NSXMLDocument *sdefXML = [[NSXMLDocument alloc] initWithContentsOfURL:[NSURL fileURLWithPath:[[NSBundle mainBundle] pathForResource:@"CHNYC PlugIn Demo 2" ofType:@"sdef"]] options:NSXMLNodeOptionsNone error:&error];
	if (sdefXML) {
		NSXMLElement *sdefXMLRootElement = [sdefXML rootElement];
		for (CHNYCPlugIn *plugIn in [CHNYCPlugIn loadedPlugIns]) {
			NSString *plugInSdefPath = [plugIn sdefPath];
			if (!plugInSdefPath) continue;
			NSXMLDocument *plugInSdefXML = [[NSXMLDocument alloc] initWithContentsOfURL:[NSURL fileURLWithPath:plugInSdefPath] options:NSXMLNodeOptionsNone error:&error];
			if (plugInSdefXML) {
				for (NSXMLNode *child in [[plugInSdefXML rootElement] children]) {
					[child detach];
					[sdefXMLRootElement addChild:child];
				}
				[plugInSdefXML release];
			} else {
				if (error) NSLog(@"Error loading sdef file %@: %@", plugInSdefPath, [error localizedDescription]);
			}
		}
		[sdefXML setCharacterEncoding:(NSString *)CFStringConvertEncodingToIANACharSetName(kCFStringEncodingUTF8)];
		[sdefXML setStandalone:NO];
		combinedSdefDescriptor = [NSAppleEventDescriptor descriptorWithDescriptorType:typeUTF8Text data:[sdefXML XMLDataWithOptions:NSXMLDocumentValidate]];
		[sdefXML release];
	} else {
		if (error) NSLog(@"Error loading base sdef file: %@", [error localizedDescription]);
	}
	if (combinedSdefDescriptor) [replyEvent setDescriptor:combinedSdefDescriptor forKeyword:keyDirectObject];
}

@end
