//
//  AppDelegate.m
//  CHNYC PlugIn Demo
//
//  Created by Jonathan Nathan on 10/11/11.
//  Copyright 2011 JNSoftware LLC. All rights reserved.
//

#import "AppDelegate.h"
#import <CHNYCPlugInDemo/CHNYCPlugInDemo.h>

static NSUserDefaults *standardUserDefaults = nil;

@implementation AppDelegate

@synthesize window, arrayController, allPlugInsArrayController, allPlugInsWindow, allPlugInsTableView;

+ (void)initialize {
	static BOOL isInitialzed = NO;
	if (!isInitialzed) {
		standardUserDefaults = [NSUserDefaults standardUserDefaults];
		NSDictionary *appDefaults = [NSDictionary dictionaryWithObjectsAndKeys:
									 [NSNumber numberWithInteger:0], @"lastSelectionIndex", 
									 [NSArray arrayWithObjects:@"com.jonn8.EnglishCHNYCDemoPlugIn", nil], @"loadedPlugIns", 
									 [NSNumber numberWithBool:YES], @"reloadPlugIns", 
									 nil];
		[standardUserDefaults registerDefaults:appDefaults];
		[standardUserDefaults synchronize];
		isInitialzed = YES;
	}
}

- (void)plugInsUpdated {
	[arrayController setContent:[CHNYCPlugIn loadedPlugIns]];
	[arrayController setSortDescriptors:[NSArray arrayWithObjects:[[[NSSortDescriptor alloc] initWithKey:@"displayName" ascending:YES] autorelease], nil]];
	[arrayController rearrangeObjects];
}

- (void)applicationDidFinishLaunching:(NSNotification *)notification {
	[CHNYCPlugIn preloadPlugIns];
	if ([standardUserDefaults boolForKey:@"reloadPlugIns"]) {
		for (NSString *identifier in [standardUserDefaults objectForKey:@"loadedPlugIns"]) {
			[CHNYCPlugIn loadPlugIn:identifier];
		}
	} else {
		[CHNYCPlugIn loadPlugIn:@"com.jonn8.EnglishCHNYCDemoPlugIn"];
	}
	[[NSAppleEventManager sharedAppleEventManager] setEventHandler:self andSelector:@selector(handleGetSdefEvent:withReplyEvent:) forEventClass:'ascr' andEventID:'gsdf'];
	if (![standardUserDefaults objectForKey:[NSString stringWithFormat:@"NSWindow Frame %@", [window frameAutosaveName]]]) [window center];
	[self plugInsUpdated];
	[arrayController setSelectionIndex:[standardUserDefaults integerForKey:@"lastSelectionIndex"]];
	[window makeKeyAndOrderFront:nil];
}

- (void)applicationWillTerminate:(NSNotification *)notification {
	[standardUserDefaults setInteger:[arrayController selectionIndex] forKey:@"lastSelectionIndex"];
	[standardUserDefaults setObject:[[CHNYCPlugIn loadedPlugIns] valueForKey:@"identifier"] forKey:@"loadedPlugIns"];
	[standardUserDefaults synchronize];
}

- (BOOL)applicationShouldTerminateAfterLastWindowClosed:(NSApplication *)application {
	return YES;
}

- (IBAction)sayHello:(id)sender {
	CHNYCPlugIn *plugIn = (CHNYCPlugIn *)[[arrayController selectedObjects] lastObject];
	if (plugIn) [plugIn sayHello];
}

- (void)handleGetSdefEvent:(NSAppleEventDescriptor *)event withReplyEvent:(NSAppleEventDescriptor *)replyEvent {
	NSAppleEventDescriptor *combinedSdefDescriptor = nil;
	NSError *error = nil;
	NSXMLDocument *sdefXML = [[NSXMLDocument alloc] initWithContentsOfURL:[NSURL fileURLWithPath:[[NSBundle mainBundle] pathForResource:@"CHNYC PlugIn Demo 3" ofType:@"sdef"]] options:NSXMLNodeOptionsNone error:&error];
	if (sdefXML) {
		NSXMLElement *sdefXMLRootElement = [sdefXML rootElement];
		for (CHNYCPlugIn *plugIn in [CHNYCPlugIn loadedPlugIns]) {
			NSString *plugInSdefPath = [plugIn sdefPath];
			if (!plugInSdefPath) continue;
			NSXMLDocument *plugInSdefXML = [[NSXMLDocument alloc] initWithContentsOfURL:[NSURL fileURLWithPath:plugInSdefPath] options:NSXMLNodeOptionsNone error:&error];
			if (plugInSdefXML) {
				for (NSXMLNode *child in [[plugInSdefXML rootElement] children]) {
					[child detach];
					[sdefXMLRootElement addChild:child];
				}
				[plugInSdefXML release];
			} else {
				if (error) NSLog(@"Error loading sdef file %@: %@", plugInSdefPath, [error localizedDescription]);
			}
		}
		[sdefXML setCharacterEncoding:(NSString *)CFStringConvertEncodingToIANACharSetName(kCFStringEncodingUTF8)];
		[sdefXML setStandalone:NO];
		combinedSdefDescriptor = [NSAppleEventDescriptor descriptorWithDescriptorType:typeUTF8Text data:[sdefXML XMLDataWithOptions:NSXMLDocumentValidate]];
		[sdefXML release];
	} else {
		if (error) NSLog(@"Error loading base sdef file: %@", [error localizedDescription]);
	}
	if (combinedSdefDescriptor) [replyEvent setDescriptor:combinedSdefDescriptor forKeyword:keyDirectObject];
}

- (IBAction)showAllPlugIns:(id)sender {
	if ([[allPlugInsArrayController content] count] == 0) {
		NSMutableArray *allPlugIns = [NSMutableArray array];
		NSArray *loadedPlugIns = [[CHNYCPlugIn loadedPlugIns] valueForKey:@"identifier"];
		for (NSString *identifier in [[CHNYCPlugIn allPlugIns] allKeys]) {
			NSMutableDictionary *plugIn = [NSMutableDictionary dictionaryWithDictionary:[[CHNYCPlugIn allPlugIns] objectForKey:identifier]];
			[plugIn setObject:identifier forKey:@"identifier"];
			[plugIn setObject:(([loadedPlugIns containsObject:identifier]) ? @"✓" : @"-") forKey:@"loaded"];
			[allPlugIns addObject:plugIn];
		}
		[allPlugInsArrayController setContent:allPlugIns];
		[allPlugInsArrayController setSortDescriptors:[NSArray arrayWithObjects:[[[NSSortDescriptor alloc] initWithKey:@"name" ascending:YES] autorelease], nil]];
		[allPlugInsArrayController rearrangeObjects];
		[allPlugInsArrayController setSelectionIndex:0];
		[allPlugInsTableView reloadData];
		[allPlugInsTableView setDoubleAction:@selector(loadPlugIn:)];
	}
	if (![standardUserDefaults objectForKey:[NSString stringWithFormat:@"NSWindow Frame %@", [allPlugInsWindow frameAutosaveName]]]) [allPlugInsWindow center];
	[allPlugInsWindow makeKeyAndOrderFront:nil];
}

- (IBAction)loadPlugIn:(id)sender {
	NSMutableDictionary *plugIn = [[allPlugInsArrayController selectedObjects] lastObject];
	if ([CHNYCPlugIn loadPlugIn:[plugIn objectForKey:@"identifier"]]) {
		[plugIn setObject:@"✓" forKey:@"loaded"];
		[allPlugInsTableView reloadData];
		[self plugInsUpdated];
	}
}

@end
